local addon, data = ...

local shardName = nil
local playerName = nil

local gadgetIndex = 0
local CURLOOKUP = {}
local activeCur = {}
local activeCurID={}

local shardBalances={}

local _CLNeeded = true
local _GCNeeded = false
local _GC = {}

local popup = nil

local function FormatSep(a)
	if a >= 1000 then
		local s = ""
		local f = true
		while a>1000 do
			nx = math.floor(a/1000)
			if f then
				s = string.format("%03d", (a-(1000*nx)))
				f = false
			else
				s = string.format("%03d,%s", (a-(1000*nx)), s)
			end
			a=nx
		end
		s=a..","..s
		return s
	else
		return tostring(a)
	end
end

local function formatcur(i,v)
	if i == "coin" then
		local p = math.floor(v/10000)
		local a = v-(10000*p)
		local g = math.floor(a/100)
		local s = a-(100*g)
		return string.format("%s:%02d.%02d", FormatSep(p),g,s)
	else
		return string.format("%s", FormatSep(v))
	end
end

local function showPopup(cn)
	local curID = CURLOOKUP[cn]
	local idx=1
	local nw=0
	local bw=0
	for k,v in pairs(Gadgets_Currency_Totals[shardName]) do
		popup[idx].name:SetText(k)
		popup[idx].balance:SetText(formatcur(curID, v[curID] or 0))
		nw=math.max(nw, popup[idx].name:GetWidth())
		bw=math.max(bw, popup[idx].balance:GetWidth())
		popup[idx].name:SetVisible(true)
		idx=idx+1
	end
	
	for x=1, idx-1 do
		popup[x].name:SetWidth(nw)
		popup[x].balance:SetWidth(bw)
	end
	
	for x=idx, 6 do
		popup[x].name:SetVisible(false)
	end
end

local function Create(configuration)
	local rfHeight = 70

	local curframe = UI.CreateFrame("Frame", WT.UniqueName("currency"), WT.Context)	
	curframe.curIcon = UI.CreateFrame("Texture", WT.UniqueName("currency"), curframe)	
	curframe.curIcon:SetPoint("TOPLEFT", curframe, "TOPLEFT", 0,0)
	curframe.curText = UI.CreateFrame("Text", WT.UniqueName("currency"), curframe)
	curframe.curText:SetFontSize(12)
	curframe.curText:SetPoint("CENTERLEFT", curframe.curIcon, "CENTERRIGHT", 2,0)
	curframe.delta = UI.CreateFrame("Text", WT.UniqueName("currency"), curframe)
	curframe.delta:SetFontSize(12)
	curframe.delta:SetPoint("TOPLEFT", curframe.curText, "TOPRIGHT", 2,0)
	curframe.delta:SetVisible(false)
	
	function curframe.curIcon.Event:MouseIn()
		if configuration.shard then
			popup[1].name:SetPoint("BOTTOMRIGHT", curframe.curIcon, "CENTER", 0,0)
			showPopup(configuration.curName)
		end
	end
	
	function curframe.curIcon.Event:MouseOut()
		popup[1].name:SetVisible(false)
	end
	
	if configuration.diff == nil then
		configuration.diff = true
	end

	curframe.showdiff = configuration.diff
	
	table.insert(_GC, {cn=configuration.curName, f=curframe, c=configuration})
	
	if popup == nil then
		popup = {}
		for x = 1, 6 do
			popup[x] = {}
			if x == 1 then
				popup[x].name = UI.CreateFrame("Text", "popup.name", WT.Context)
				popup[x].name:SetVisible(false)
			else
				popup[x].name = UI.CreateFrame("Text", "popup.name", popup[x-1].name)
			end
			popup[x].balance = UI.CreateFrame("Text", "popup.balance", popup[x].name)
			popup[x].name:SetFontSize(12)
			popup[x].name:SetLayer(100)
			popup[x].name:SetBackgroundColor(0,0,0)
			popup[x].balance:SetFontSize(12)
			popup[x].balance:SetLayer(100)
			popup[x].balance:SetBackgroundColor(0,0,0)
			
			if x == 1 then
				popup[x].balance:SetPoint("TOPLEFT", popup[x].name, "TOPRIGHT", 0,0)
			else
				popup[x].name:SetPoint("BOTTOMLEFT", popup[x-1].name, "TOPLEFT", 0,0)
				popup[x].balance:SetPoint("BOTTOMLEFT", popup[x-1].balance, "TOPLEFT", 0,0)
			end
		end
	end
	
	
	_GCNeeded = true
	return curframe
end

local dialog = false

local function ConfigDialog(container)	
	local allcurrency = {}
	for k,v in pairs(Inspect.Currency.Detail(Inspect.Currency.List())) do
		table.insert(allcurrency, { text=v.name, value=v.name })
		CURLOOKUP[v.name] = k
	end
	table.sort(allcurrency, function(a,b) return a.text < b.text end)
	dialog = WT.Dialog(container)
		:Label("Currency shows current balance for selected currency, along with changes this session.")
		:Combobox("curName", "Currency to track", "Platinum, Gold, Silver",
			allcurrency
			, false)
		:Checkbox("diff", "Show changes", true)
		:Checkbox("mbags", "Interface with MoneyBags", true)
		:Checkbox("shard", "Show TOTAL balance for all characters on shard", false)
end

local function GetConfiguration()
	return dialog:GetValues()
end

local function SetConfiguration(config)
	dialog:SetValues(config)
end

local function Event_Currency(e)
	for k,v in pairs(e) do
		Gadgets_Currency_Totals[shardName][playerName][k] = v
		if activeCurID[k] then
			for c,d in pairs(activeCur) do
				if d.id == k then
					local adj = 0
					if d.shard then
						adj = shardBalances[d.id] or 0
					end
					d.f.curText:SetText(formatcur(k,v+adj))
					if d.f.showdiff then
						if v > d.f.start then
							d.f.delta:SetText(formatcur(k,(v-d.f.start)))
							d.f.delta:SetFontColor(0,1,0)
							d.f.delta:SetVisible(true)
							d.f:SetWidth(d.f.h + d.f.curText:GetWidth() + d.f.delta:GetWidth() + 4)
						elseif v < d.f.start then
							d.f.delta:SetText("-"..formatcur(k,(d.f.start-v)))
							d.f.delta:SetFontColor(1,0,0)
							d.f.delta:SetVisible(true)
							d.f:SetWidth(d.f.h + d.f.curText:GetWidth() + d.f.delta:GetWidth() + 4)
						else
							d.f.delta:SetVisible(false)
							d.f:SetWidth(d.f.h + d.f.curText:GetWidth() + 2)
						end
					else
						d.f.delta:SetVisible(false)
						d.f:SetWidth(d.f.h + d.f.curText:GetWidth() + 2)					
					end
				end
			end
		end
	end
end

WT.Gadget.RegisterFactory("Currency",
	{
		name="Currency",
		description="Shows current and changes to a currency",
		author="Adelea@Icewatch-EU",
		version="1.10r1",
		iconTexAddon=addon.identifier,
		iconTexFile="img/platinum.png",
		["Create"] = Create,
		["ConfigDialog"] = ConfigDialog,
		["GetConfiguration"] = GetConfiguration, 
		["SetConfiguration"] = SetConfiguration, 
	})

local function Event_Addon_SavedVariables_Load_End(a)
	if a ~= addon.identifier then
		return
	end
	
	Gadgets_Currency_Totals = Gadgets_Currency_Totals or {}
	
	if shardName == nil then
		shardName = Inspect.Shard().name
	end
	
	Gadgets_Currency_Totals[shardName] = Gadgets_Currency_Totals[shardName] or {}
	
	playerName = Inspect.Unit.Detail("player").name

	for k,v in pairs(Gadgets_Currency_Totals[shardName]) do
		if k ~= playerName then
			for k1,v1 in pairs(v) do
				if shardBalances[k1] == nil then
					shardBalances[k1] = 0
				end
				shardBalances[k1] = shardBalances[k1] + v1
			end
		end
	end
	
end
	
local function Event_System_Update_Begin()
	if _CLNeeded then
		Gadgets_Currency_Totals[shardName][playerName] = {}
		local cnt=0
		for k,v in pairs(Inspect.Currency.Detail(Inspect.Currency.List())) do
			cnt = cnt+1
			CURLOOKUP[v.name] = k
			Gadgets_Currency_Totals[shardName][playerName][k] = v.stack
		end
		if cnt > 0 then
			_GCNeeded = true
			_CLNeeded = false
		end
	end
	if (_CLNeeded == false) and _GCNeeded then
		for k,v in pairs(_GC) do
			local curID = CURLOOKUP[v.cn]
			local d = Inspect.Currency.Detail(curID)	
			activeCurID[curID] = true
			
			if d.icon then
				v.f.curIcon:SetTexture("Rift", d.icon)
			else
				v.f.curIcon:SetTexture(addon.identifier, "img/platinum.png")
			end
			
			local start = 0
			local dstart= 0
			local dx = 0
							
			if v.c.mbags == nil then
				v.c.mbags = false
			end

			if (MoneyBags_Settings == nil) or (MoneyBags_Settings ~= nil and ((v.c.mbags or false) == false)) then
				start = d.stack
				dstart = d.stack
			else
				start = d.stack
				if MoneyBags_Settings.history[os.date("%Y%m%d")] == nil then
					dstart = d.stack
				else
					dstart = MoneyBags_Settings.history[os.date("%Y%m%d")][v.cn].start
				end
			end
			
			local adj = 0
			if v.c.shard == nil then
				v.c.shard = false
			end
			
			if v.c.shard then
				adj = shardBalances[curID] or 0
			end

			if v.f.showdiff then
				if start > dstart then
					v.f.delta:SetText(formatcur(curID, (start - dstart)))
					dx = v.f.delta:GetWidth()+2
					v.f.delta:SetFontColor(0,1,0)
					v.f.delta:SetVisible(true)
				elseif start < dstart then
					v.f.delta:SetText("-"..formatcur(curID, (dstart - start)))
					dx = v.f.delta:GetWidth()+2
					v.f.delta:SetFontColor(1,0,0)
					v.f.delta:SetVisible(true)
				else
					v.f.delta:SetVisible(false)
				end
			else
				v.f.delta:SetVisible(false)
			end
			
			v.f.start = start
			
			v.f.curText:SetText(formatcur(curID, start+adj))
			
			v.f.h = v.f.curText:GetHeight()
			v.f:SetHeight(v.f.h)
			v.f:SetWidth(v.f.h+v.f.curText:GetWidth()+2+dx)
			v.f.curIcon:SetHeight(v.f.h)
			v.f.curIcon:SetWidth(v.f.h)
			
			table.insert(activeCur, { f=v.f, id=curID, shard=v.c.shard })
		end
		_GCNeeded = false
		_GC = {}
	end
end

table.insert(Event.Currency, { Event_Currency, addon.identifier, "Event.Currency" })
table.insert(Event.System.Update.Begin, { Event_System_Update_Begin, addon.identifier, "Event.System.Update.Begin"})
table.insert(Event.Addon.SavedVariables.Load.End, { Event_Addon_SavedVariables_Load_End, addon.identifier, "Event.Addon.SavedVariables.Load.End" })