local toc, data = ...
local AddonId = toc.identifier

-- Local config options ---------------------------------------------------------
local miniRaidFrameWidth = 90
local miniRaidFrameTopBarHeight = 21
local miniRaidFrameBottomBarHeight = 6
local miniRaidFrameMargin = 2
local miniRaidFrameHeight = miniRaidFrameTopBarHeight + miniRaidFrameBottomBarHeight
local miniRaidFrameFadeRange = 35
local miniRaidFrameFadeRangeSqu = miniRaidFrameFadeRange * miniRaidFrameFadeRange
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local MiniRaidFrame = WT.UnitFrame:Template("MiniRaidFrame")
MiniRaidFrame.Configuration.Name = "Compact Raid Frame"
MiniRaidFrame.Configuration.RaidSuitable = true
MiniRaidFrame.Configuration.FrameType = "Frame"
MiniRaidFrame.Configuration.Width = miniRaidFrameWidth
MiniRaidFrame.Configuration.Height = miniRaidFrameHeight + 3
MiniRaidFrame.Configuration.Resizable = { miniRaidFrameWidth, miniRaidFrameHeight + 3, 300, 100 }
---------------------------------------------------------------------------------

-- Override the buff filter to hide some buffs ----------------------------------
local buffPriorities = 
{
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
}
function MiniRaidFrame:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end
---------------------------------------------------------------------------------

function MiniRaidFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" }
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				height=-3, width=miniRaidFrameWidth, alpha=0.5,
				color={r=0,g=0,b=0,a=0.3}, 
			},
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frameBackdrop", layer=20, visibilityBinding="blockedOrRanged",
				color={r=0,g=0,b=0},alpha=0.8,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" }
				},
			}, 
			{
				-- Generic Element Configuration
				id="barResource", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="BOTTOMLEFT", element="frame", targetPoint="BOTTOMLEFT", offsetX=1, offsetY=0 },
					{ point="RIGHT", element="frame", targetPoint="RIGHT", offsetX=-1 }
				},
				-- Type Specific Element Configuration
				binding="resourcePercent", height=miniRaidFrameBottomBarHeight, colorBinding="resourceColor",
				texAddon=AddonId, texFile="img/WinBrV.png",
				backgroundColor={r=0, g=0, b=0, a=1}
			},
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=3 },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="TOPRIGHT" }
				},
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", height=miniRaidFrameTopBarHeight, colorBinding="callingColor",
				texAddon=AddonId, texFile="img/WinBrV.png", 
				backgroundColor={r=0, g=0, b=0, a=0}
			},
			{
				-- Generic Element Configuration
				id="frameAggro", type="Frame", parent="frameBackdrop", layer=10,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=0 },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint="TOPRIGHT" }
				},
				color={r=0,g=0,b=0,a=1}, colorBinding="aggroColor",
			},
			{
				-- Generic Element Configuration
				id="frameColor", type="Frame", parent="frameBackdrop", layer=15, alpha=0.3,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=3 },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="TOPRIGHT" }
				},
				colorBinding="callingColor", 
			},
			{
				-- Generic Element Configuration
				id="frameColor2", type="Frame", parent="frameBackdrop", layer=15, alpha=0.3,
				attach = { 
					{ point="TOPLEFT", element="barHealth", targetPoint="BOTTOMLEFT", offsetX=1, offsetY=0 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" }
				},
				colorBinding="resourceColor", 
			},
			{
				-- Generic Element Configuration
				id="imgRole", type="ImageSet", parent="frame", layer=40,
				attach = {{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT", offsetX=0, offsetY=0 }},
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/Roles12.png", nameBinding="role", cols=1, rows=5, 
				names = { ["tank"] = 0, ["heal"] = 1, ["dps"] = 2, ["support"] = 3 }, defaultIndex = "hide"
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=40, alpha=1,
				attach = {{ point="CENTERLEFT", element="imgRole", targetPoint="CENTERRIGHT", offsetX=0, offsetY=0 }},
				visibilityBinding="name", 
				-- Type Specific Element Configuration
				text="{nameMini}", default="", fontSize=12,
				color={r=1,g=1,b=1,a=1},
			},
			{
				-- Generic Element Configuration
				id="labelHealth", type="Label", parent="frame", layer=40, alpha=1,
				attach = {{ point="TOPLEFT", element="labelName", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=-6 }},
				visibilityBinding="healthtext", 
				-- Type Specific Element Configuration
				text="{healthPercent}%", default="", fontSize=10,
				color={r=1,g=1,b=1,a=1},
			},
			{
				-- Generic Element Configuration
				id="labelStatus", type="Label", parent="frame", layer=40, alpha=1,
				attach = {{ point="TOPLEFT", element="labelName", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=-6 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{raidStatusMini}", default="", fontSize=10,
			},
			{
				-- Generic Element Configuration
				id="labelMark", type="Label", parent="frameBackdrop", layer=20,
				attach = {{ point="TOPRIGHT", element="frameBackdrop", targetPoint="TOPRIGHT", offsetX=1.5, offsetY=-3.5 }},
				visibilityBinding="mark",alpha=0.85,
				-- Type Specific Element Configuration
				text="{mark}", default="X", fontSize=22
			},
			{
				-- Generic Element Configuration
				id="imgReady", type="ImageSet", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frame", targetPoint="CENTER" }},
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/wtReady.png", nameBinding="readyStatus", cols=1, rows=2, 
				names = { ["ready"] = 0, ["notready"] = 1 }, defaultIndex = "hide",
			},
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=80,
				attach = {{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=0 }},
				rows=1, cols=5, iconSize=13, iconSpacingHorizontal=1, iconSpacingVertical=1, borderThickness=1, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", borderColor={r=1,g=1,b=1,a=1},
				sweepOverlay=false,
			},
			
		}
	}
	
	for idx,element in ipairs(template.elements) do
		if options.excludeBuffs and element.type=="BuffPanel" then
			-- Don't add buff panels if excluding them
		else
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self:SetMouseMasking("limited")
	if options.clickToTarget then
		self.Event.LeftClick = "target @" .. self.UnitSpec
	end
	if options.contextMenu then
		self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
	end
end

WT.Unit.CreateVirtualProperty("nameMini", { "name" },
	function(unit)
		if unit.name and unit.name:len() > 11 then
			nameMini = unit.name:sub(1, 8) .. "..."
			return nameMini
		else
			return unit.name
		end
	end)

WT.Unit.CreateVirtualProperty("raidStatusMini", { "offline", "afk", "health" },
	function(unit)
		if unit.offline then
			return "OFFLINE"
		elseif unit.afk then
			return "AFK"
		elseif unit.health and unit.health == 0 then
			return "DEAD"
		else
			return nil
		end
	end)


WT.Unit.CreateVirtualProperty("healthtext", { "health", "afk", "dead", "offline" },
	function(unit)
		if unit.afk then
			return nil
		elseif unit.dead then
			return nil
		elseif unit.offline then
			return nil
		else
			return unit.health
		end
	end)