local nkGenie, privateVars = ...

nkGenie.Tools = {}
nkGenie.Events = {}	

nkGenie.ExtButton = {}
nkGenie.ExtCheckbox = {}
nkGenie.ExtCombo = {}
nkGenie.ExtDialog = {}
nkGenie.ExtFrame = {}
nkGenie.ExtGrid = {}
nkGenie.ExtItemTooltip = {}
nkGenie.ExtMask = {}
nkGenie.ExtMenu = {}
nkGenie.ExtRadioButton = {}
nkGenie.ExtRiftButton = {}
nkGenie.ExtRiftWindow = {}
nkGenie.ExtSlider = {}
nkGenie.ExtScrollBox = {}
nkGenie.ExtTabPane = {}
nkGenie.ExtText = {}
nkGenie.ExtTextField = {}
nkGenie.ExtTexture = {}
nkGenie.ExtWindow = {}
nkGenie.ExtProgressBar = {}

privateVars.UIElements = { 
	nkExtButton		= nkGenie.ExtButton,
	nkExtCheckbox 	= nkGenie.ExtCheckbox,
	nkExtCombo		= nkGenie.ExtCombo,
	nkExtDialog		= nkGenie.ExtDialog,
	nkExtFrame		= nkGenie.ExtFrame,
	nkExtGrid		= nkGenie.ExtGrid,
	nkExtItemTooltip= nkGenie.ExtItemTooltip,
	nkExtMask		= nkGenie.ExtMask,
	nkExtMenu		= nkGenie.ExtMenu,
	nkExtRadioButton= nkGenie.ExtRadioButton,
	nkExtRiftButton = nkGenie.ExtRiftButton,
	nkExtRiftWindow = nkGenie.ExtRiftWindow,
	nkExtSlider		= nkGenie.ExtSlider,
	nkExtScrollBox	= nkGenie.ExtScrollBox,
	nkExtTabPane	= nkGenie.ExtTabPane,
	nkExtText		= nkGenie.ExtText,
	nkExtTextField  = nkGenie.ExtTextField,
	nkExtTexture	= nkGenie.ExtTexture,
	nkExtWindow		= nkGenie.ExtWindow,
	nkExtProgressBar= nkGenie.ExtProgressBar
}

privateVars.coRoutines = {}

nkGenie.context = UI.CreateContext("nkGenie")

local _UICreateFrame = UI.CreateFrame

UI.CreateFrame = function(type, name, parent, settings)
	
	if privateVars.UIElements[type] ~= nil then
		local element = privateVars.UIElements[type]:new (name, parent, settings)
		if settings ~= nil then element:update(settings) end
		return element
	else
		return _UICreateFrame(type, name, parent)
	end

end

function nkGenie.Tools.checkProps(element, attribute, value)

	if value == nil then return false end
	if element.props == nil then return true end
	
	if string.find(attribute, '%.') == nil then	
		if element.props[attribute] == nil then return true end
		if element.props[attribute] ~= value then return true end
	else
		local tempArray = attribute:split('%.')
		local tempValue = element.props
		
		for idx = 1, #tempArray, 1 do
			if tempValue[tempArray[idx]] == nil then return true end
			if idx == #tempArray and tempValue[tempArray[idx]] ~= value then return true end
			tempValue = tempValue[tempArray[idx]]
		end
	end
	
	return false

end

function nkGenie.Tools.checkAnchorUpdate(self, newAnchors)

	local updateAnchor = false
	
	if self.props.anchors == nil or #self.props.anchors ~= #newAnchors then
		updateAnchor = true
	else			
		for idx = 1, #newAnchors, 1 do
			if	newAnchors[idx].from ~= self.props.anchors[idx].from or 
				newAnchors[idx].object:GetName() ~= self.props.anchors[idx].object:GetName() or 
				newAnchors[idx].to ~= self.props.anchors[idx].to or 
				newAnchors[idx].x ~= self.props.anchors[idx].x or 
				newAnchors[idx].y ~= self.props.anchors[idx].y then
				updateAnchor = true
				break
			end
		end
	end

	return updateAnchor
	
end

function nkGenie.Tools:HexToRGB(hexValue)

	if type(hexValue) ~= "string" then return nil end
	if string.len(hexValue) < 6 then return nil end
	
	local retValue = {}
	
	for i = 1, 6, 2 do		
		local subString = string.sub(hexValue, i, (i+1))
		local decValue = tonumber(subString, 16)
		table.insert (retValue, (1 / 255 * decValue))
	end
	
	return retValue
	
end

function nkGenie.Tools:RGBToHex(red, green, blue)

	local redHex = nkGenie.Tools:DecToHex(red)
	local greenHex = nkGenie.Tools:DecToHex(green)
	local blueHex = nkGenie.Tools:DecToHex(blue)
	
	local retValue = ''
	
	if red == 0 then
		retValue = '00'
	elseif string.len(redHex) == 1 then
		retValue = '0' .. redHex
	else
		retValue = redHex
	end

	if green == 0 then
		retValue = retValue .. '00'
	elseif string.len(greenHex) == 1 then
		retValue = retValue .. '0' .. greenHex
	else
		retValue = retValue .. greenHex
	end
	
	if blue == 0 then
		retValue = retValue .. '00'
	elseif string.len(blueHex) == 1 then
		retValue = retValue .. '0' .. blueHex
	else
		retValue = retValue .. blueHex
	end

	return retValue
	
end

function nkGenie.Tools:DecToHex(value)

    local b,k,result,i,d=16,"0123456789ABCDEF","",0
	
    while value > 0 do
        i=i+1
        value,d = math.floor(value/b),math.mod(value,b)+1
        result=string.sub(k,d,d)..result
    end
	
    return result
	
end

function nkGenie.Tools:ColorAdjust(color, adjust)

	if type(color) ~= "string" then return nil end
	if string.len(color) < 6 then return nil end
	local newColor = ''
	
	for i = 1, 6, 2 do		
		local subString = string.sub(color, i, (i+1))
		local decValue = tonumber(subString, 16)
		decValue = decValue * adjust
		local hex = nkGenie.Tools:DecToHex(decValue)
		if string.len(hex) == 1 then hex = '0' .. hex end			
		newColor = newColor .. hex
	end
	
	return newColor

end

function nkGenie.Tools.round (num, idp)

	if num == nil then return nil end

	local mult = 10^(idp or 0)
	return math.floor(tonumber(num) * mult + 0.5) / mult
end

function nkGenie.Tools.tableCopy (tableToCopy)

    local lookup_table = {}
    local function _copy(tableToCopy)
        if type(tableToCopy) ~= "table" then
            return tableToCopy
        elseif lookup_table[tableToCopy] then
            return lookup_table[tableToCopy]
        end
        local new_table = {}
        lookup_table[tableToCopy] = new_table
        for index, value in pairs(tableToCopy) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(tableToCopy))
    end
    return _copy(tableToCopy)
end

function nkGenie.Events.updateHandler()

	if #privateVars.coRoutines > 0 then
		for idx = 1, #privateVars.coRoutines, 1 do
			if privateVars.coRoutines[idx].active == true then
				local status, value = coroutine.resume(privateVars.coRoutines[idx].func, privateVars.coRoutines[idx].para1, privateVars.coRoutines[idx].para2)
				
				if status == false then 
					print ('error in coroutine: ' .. value)
					privateVars.coRoutines[idx].active = false
				elseif value >= privateVars.coRoutines[idx].counter or status == false then
					privateVars.coRoutines[idx].active = false
				end
			end
		end
		
		if privateVars.coRoutines[1].active == false then
			if privateVars.coRoutines[1].callBack ~= nil then privateVars.coRoutines[1].callBack() end
			table.remove(privateVars.coRoutines, 1)
		end
	end	

end

table.insert(Event.System.Update.Begin, { nkGenie.Events.updateHandler, "nkGenie", "updateHandler" })