local nkGenie, privateVars = ...

function nkGenie.ExtMask:new(name, parent, settings)

	local this = {}
	this.name = name
	this.parent = parent
	this.props = {}
	
	local element = UI.CreateFrame("Mask", name, parent)
		
	this.element = element
	setmetatable(this, self)
	self.__index = self
	return this
	
end

function nkGenie.ExtMask:checkProps(attribute, value) return nkGenie.Tools.checkProps(self, attribute, value) end
function nkGenie.ExtMask:checkAnchorUpdate(newAnchors) return nkGenie.Tools.checkAnchorUpdate(self, newAnchors) end

function nkGenie.ExtMask:update (settings)

	if settings == nil then return end
	
	if settings.template ~= nil then
		for k, v in pairs (settings.template) do if settings[k] == nil then settings[k] = v end end
		settings.template = nil
	end	
	
	if self:checkProps('width', settings.width) == true then self.element:SetWidth(settings.width) end
	if self:checkProps('height', settings.height) == true then self.element:SetHeight(settings.height) end
	
	if settings.anchors ~= nil and self:checkAnchorUpdate (settings.anchors) == true then
		if self.anchors ~= nil then
			for k, anchor in pairs (self.anchors) do self.element:ClearPoint (anchor.from) end
		end
	
		for k, anchor in pairs(settings.anchors) do
			if anchor.x ~= nil and anchor.y ~= nil then
				self.element:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				self.element:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end
	end	
	
	if self:checkProps('layer', settings.layer) == true then self.element:SetLayer(settings.layer) end
	if self:checkProps('strata', settings.strata) == true then self.element:SetStrata(settings.strata) end
	if self:checkProps('visible', settings.visible) == true then self.element:SetVisible(settings.visible) end
	
	for k, v in pairs (settings) do self.props[k] = v end
	
end

function nkGenie.ExtMask:getElement() return self.element end

function nkGenie.ExtMask:GetVisible() 
	if self.props.visible == nil then self.props.visible = self:getElement(false):GetVisible() end
	return self.props.visible
end

function nkGenie.ExtMask:SetVisible(flag) 
	if self:checkProps('visible', flag) == true then 
		self:getElement(false):SetVisible(flag) 
		self.props.visible = flag
	end
end
	
function nkGenie.ExtMask:SetParent(parent) self:getElement(false):SetParent(parent) end

function nkGenie.ExtMask:GetWidth() 
	if self.props.width == nil then self.props.width = self:getElement(false):GetWidth() end
	return self.props.width
end

function nkGenie.ExtMask:GetHeight() 
	if self.props.height == nil then self.props.height = self:getElement(false):GetHeight() end
	return self.props.height
end

function nkGenie.ExtMask:SetSecureMode(mode) 
	if self:checkProps('securemode', mode) == true then 
		self:getElement(false):SetSecureMode(mode) 
		self.props.securemode = mode
	end
end

function nkGenie.ExtMask:SetStrata(strata) 
	if self:checkProps('strata', strata) == true then 
		self:getElement(false):SetStrata(strata) 
		self.props.strata = strata
	end
end

function nkGenie.ExtMask:SetLayer(layer) 
	if self:checkProps('layer', layer) == true then 
		self:getElement(false):SetLayer(layer) 
		self.props.layer = layer
	end
end